@@ -29,6 +29,7 @@ import("core.project.project")
 import("lib.detect.find_file")
 import("lib.detect.find_tool")
 import("package.tools.ninja")
+import("package.tools.msbuild")
 import("detect.sdks.find_emsdk")
 import("private.utils.toolchain", {alias = "toolchain_utils"})
 
@@ -97,23 +98,13 @@ function _get_msvc_runenvs(package)
     return os.joinenvs(_get_msvc(package):runenvs())
 end
 
--- get vs arch
-function _get_vsarch(package)
-    local arch = package:arch()
-    if arch == 'x86' or arch == 'i386' then return "Win32" end
-    if arch == 'x86_64' then return "x64" end
-    if arch:startswith('arm64') then return "ARM64" end
-    if arch:startswith('arm') then return "ARM" end
-    return arch
-end
-
 -- get cflags from package deps
 function _get_cflags_from_packagedeps(package, opt)
     local result = {}
     for _, depname in ipairs(opt.packagedeps) do
         local dep = type(depname) ~= "string" and depname or package:dep(depname)
         if dep then
-            local fetchinfo = dep:fetch({external = false})
+            local fetchinfo = dep:fetch()
             if fetchinfo then
                 table.join2(result, _map_compflags(package, "cxx", "define", fetchinfo.defines))
                 table.join2(result, _translate_paths(_map_compflags(package, "cxx", "includedir", fetchinfo.includedirs)))
@@ -130,7 +121,7 @@ function _get_ldflags_from_packagedeps(package, opt)
     for _, depname in ipairs(opt.packagedeps) do
         local dep = type(depname) ~= "string" and depname or package:dep(depname)
         if dep then
-            local fetchinfo = dep:fetch({external = false})
+            local fetchinfo = dep:fetch()
             if fetchinfo then
                 table.join2(result, _translate_paths(_map_linkflags(package, "binary", {"cxx"}, "linkdir", fetchinfo.linkdirs)))
                 table.join2(result, _map_linkflags(package, "binary", {"cxx"}, "link", fetchinfo.links))
@@ -297,7 +288,8 @@ function _get_cmake_system_processor(package)
             x64 = "AMD64",
             x86_64 = "AMD64",
             arm = "ARM",
-            arm64 = "ARM64"
+            arm64 = "ARM64",
+            arm64ec = "ARM64EC"
         }
         return archs[package:arch()] or package:arch()
     end
@@ -356,6 +348,8 @@ function _get_configs_for_windows(package, configs, opt)
             table.insert(configs, "Win32")
         elseif package:is_arch("arm64") then
             table.insert(configs, "ARM64")
+        elseif package:is_arch("arm64ec") then
+            table.insert(configs, "ARM64EC")
         elseif package:is_arch("arm.*") then
             table.insert(configs, "ARM")
         else
@@ -875,6 +869,17 @@ function _get_configs(package, configs, opt)
     return configs
 end
 
+-- Fix pdb issue, if multiple CL.EXE write to the same .PDB file, please use /FS
+-- @see https://github.com/xmake-io/xmake/issues/5353
+function _fix_pdbdir_for_ninja(package)
+    if package:is_plat("windows") and package:has_tool("cxx", "cl") then
+        local pdbdir = "pdb"
+        if not os.isdir(pdbdir) then
+            os.mkdir(pdbdir)
+        end
+    end
+end
+
 -- get build environments
 function buildenvs(package, opt)
 
@@ -930,14 +935,8 @@ end
 
 -- do build for msvc
 function _build_for_msvc(package, configs, opt)
-    local jobs = _get_parallel_njobs(opt)
     local slnfile = assert(find_file("*.sln", os.curdir()), "*.sln file not found!")
-    local runenvs = _get_msvc_runenvs(package)
-    local msbuild = find_tool("msbuild", {envs = runenvs})
-    os.vrunv(msbuild.program, {slnfile, "-nologo", "-t:Rebuild",
-            (jobs ~= nil and format("-m:%d", jobs) or "-m"),
-            "-p:Configuration=" .. (package:is_debug() and "Debug" or "Release"),
-            "-p:Platform=" .. _get_vsarch(package)}, {envs = runenvs})
+    msbuild.build(package, {slnfile, "-t:Rebuild"})
 end
 
 -- do build for make
@@ -975,6 +974,7 @@ end
 -- do build for ninja
 function _build_for_ninja(package, configs, opt)
     opt = opt or {}
+    _fix_pdbdir_for_ninja(package)
     ninja.build(package, {}, {envs = opt.envs or buildenvs(package, opt),
         jobs = opt.jobs,
         target = opt.target})
@@ -997,17 +997,11 @@ end
 
 -- do install for msvc
 function _install_for_msvc(package, configs, opt)
-    local jobs = _get_parallel_njobs(opt)
     local slnfile = assert(find_file("*.sln", os.curdir()), "*.sln file not found!")
-    local runenvs = _get_msvc_runenvs(package)
-    local msbuild = assert(find_tool("msbuild", {envs = runenvs}), "msbuild not found!")
-    os.vrunv(msbuild.program, {slnfile, "-nologo", "-t:Rebuild", "/nr:false",
-        (jobs ~= nil and format("-m:%d", jobs) or "-m"),
-        "-p:Configuration=" .. (package:is_debug() and "Debug" or "Release"),
-        "-p:Platform=" .. _get_vsarch(package)}, {envs = runenvs})
+    msbuild.build(package, {slnfile, "-t:Rebuild", "/nr:false"})
     local projfile = os.isfile("INSTALL.vcxproj") and "INSTALL.vcxproj" or "INSTALL.vcproj"
     if os.isfile(projfile) then
-        os.vrunv(msbuild.program, {projfile, "/property:configuration=" .. (package:is_debug() and "Debug" or "Release")}, {envs = runenvs})
+        msbuild.build(package, {projfile})
         os.trycp("install/bin", package:installdir())
         os.trycp("install/lib", package:installdir()) -- perhaps only headers library
         os.trycp("install/share", package:installdir())
@@ -1059,6 +1053,7 @@ end
 -- do install for ninja
 function _install_for_ninja(package, configs, opt)
     opt = opt or {}
+    _fix_pdbdir_for_ninja(package)
     ninja.install(package, {}, {envs = opt.envs or buildenvs(package, opt),
         jobs = opt.jobs,
         target = opt.target})
@@ -1082,7 +1077,7 @@ function _get_cmake_generator(package, opt)
     opt = opt or {}
     local cmake_generator = opt.cmake_generator
     if not cmake_generator then
-        if project.policy("package.cmake_generator.ninja") then
+        if project.policy("package.cmake_generator.ninja") or package:policy("package.cmake_generator.ninja") then
             cmake_generator = "Ninja"
         end
         if not cmake_generator then
