@@ -30,6 +30,7 @@ import("net.fasturl")
 import("private.action.require.impl.package")
 import("private.action.require.impl.lock_packages")
 import("private.action.require.impl.register_packages")
+import("private.action.require.impl.actions.check", {alias = "action_check"})
 import("private.action.require.impl.actions.install", {alias = "action_install"})
 import("private.action.require.impl.actions.download", {alias = "action_download"})
 
@@ -281,7 +282,7 @@ function _fetch_packages(packages_fetch, installdeps)
         while instance == nil and #packages_pending > 0 do
             for idx, pkg in ipairs(packages_pending) do
 
-                -- all dependences has been fetched? we fetch it now
+                -- all dependencies has been fetched? we fetch it now
                 local ready = true
                 local dep_not_ready = nil
                 for _, dep in pairs(installdeps[tostring(pkg)]) do
@@ -330,7 +331,9 @@ function _fetch_packages(packages_fetch, installdeps)
             packages_fetching[index] = instance
             local oldenvs = os.getenvs()
             instance:envs_enter()
+            instance:lock()
             instance:fetch()
+            instance:unlock()
             os.setenvs(oldenvs)
 
             -- fix terminal mode to avoid some subprocess to change it
@@ -357,6 +360,14 @@ function _fetch_packages(packages_fetch, installdeps)
           isolate = true})
 end
 
+-- should fetch package?
+function _should_fetch_package(instance)
+    if instance:is_fetchonly() or not option.get("force") or
+        (option.get("shallow") and not instance:is_toplevel()) then
+        return true
+    end
+end
+
 -- should install package?
 function _should_install_package(instance)
     _g.package_status_cache = _g.package_status_cache or {}
@@ -399,7 +410,7 @@ function _install_packages(packages_install, packages_download, installdeps)
         while instance == nil and #packages_pending > 0 do
             for idx, pkg in ipairs(packages_pending) do
 
-                -- all dependences has been installed? we install it now
+                -- all dependencies has been installed? we install it now
                 local ready = true
                 local dep_not_found = nil
                 for _, dep in pairs(installdeps[tostring(pkg)]) do
@@ -469,6 +480,7 @@ function _install_packages(packages_install, packages_download, installdeps)
                 local downloaded = true
                 if packages_download[tostring(instance)] then
                     packages_downloading[index] = instance
+                    action_check(instance)
                     downloaded = action_download(instance)
                     packages_downloading[index] = nil
                 end
@@ -480,6 +492,7 @@ function _install_packages(packages_install, packages_download, installdeps)
                         assert(instance:is_precompiled(), "package(%s) should be precompiled", instance:name())
                         -- we need to disable built and re-download and re-install it
                         instance:fallback_build()
+                        action_check(instance)
                         action_download(instance)
                         action_install(instance)
                     end
@@ -672,9 +685,7 @@ function main(requires, opt)
     -- fetch and register packages (with system) from local first
     local packages_fetch = {}
     for _, instance in ipairs(packages) do
-        if instance and (instance:is_fetchonly() or
-                         not option.get("force") or
-                         (option.get("shallow") and not instance:is_toplevel())) then
+        if _should_fetch_package(instance) then
             table.insert(packages_fetch, instance)
         end
     end
